/*****************************************/
/*                                       */
/*   FUSION socket library header file   */
/*                                       */
/*****************************************/
#include <zcppcmn.h>        /* C++ support  99/9/8 */
#ifndef _FUSIONH_
#define _FUSIONH_

BEGIN_NAMESPACE_ZAURUS      /* C++ support  99/9/8 */
/***** "ccdep" *****/

#define u8  unsigned char

#define i16 short
#define u16 unsigned short

#define i32 long
#define u32 unsigned long


/***** "std" *****/

typedef struct a16 {        /* a 16-bit byte-ordered structure */
    u8  a2[2];
} a16;
typedef struct a32 {        /* a 32-bit byte-ordered structure */
    u8  a4[4];
} a32;
typedef struct a48 {        /* a 48-bit byte-ordered structure */
    u8  a6[6];
} a48;


/***** "nerrno" *****/

#define NET_ERR         80

/* General purpose errors */

#define EZERO           0               /* No error */
#define ENOERR          0               /* No error */
#define EPERM           1               /* Not owner */
#define ENOENT          2               /* No such file or directory */
#define ESRCH           3               /* No such process */
#define EINTR           4               /* Interrupted system call */
#define EIO             5               /* I/O error */
#define ENXIO           6               /* No such device or address */
#define E2BIG           7               /* Arg list too long */
#define ENOEXEC         8               /* Exec format error */
#define EBADF           9               /* Bad file number */
#define ECHILD          10              /* No children */
#define EAGAIN          11              /* No more processes */
#define ENOMEM          12              /* Not enough core */
#define EACCES          13              /* Permission denied */
#define EFAULT          14              /* Bad address */
#define ENOTBLK         15              /* Block device required */
#define EBUSY           16              /* Mount device busy */
#define EEXIST          17              /* File exists */
#define EXDEV           18              /* Cross-device link */
#define ENODEV          19              /* No such device */
#define ENOTDIR         20              /* Not a directory*/
#define EISDIR          21              /* Is a directory */
#define EINVAL          22              /* Invalid argument */
#define ENFILE          23              /* File table overflow */
#define EMFILE          24              /* Too many open files */
#define ENOTTY          25              /* Not a typewriter */
#define ETXTBSY         26              /* Text file busy */
#define EFBIG           27              /* File too large */
#define ENOSPC          28              /* No space left on device */
#define ESPIPE          29              /* Illegal seek */
#define EROFS           30              /* Read-only file system */
#define EMLINK          31              /* Too many links */
#define EPIPE           32              /* Broken pipe */
#define EDOM            33              /* Argument too large */
#define ERANGE          34              /* Result too large */
#define ENOMSG          35              /* No message of desired type */
#define EIDRM           36              /* Identifier removed */
#define ECHRNG          37              /* Channel number out of range */
#define EL2NSYNC        38              /* Level 2 not synchronized */
#define EL3HLT          39              /* Level 3 halted */
#define EL3RST          40              /* Level 3 reset */
#define ELNRNG          41              /* Link number out of range */
#define EUNATCH         42              /* Protocol driver not attached */
#define ENOCSI          43              /* No CSI structure available */
#define EL2HLT          44              /* Level 2 halted */
#define EDEADLOCK       45              /* Locking deadlock */

/* Network errors */

#define EWOULDBLOCK             (NET_ERR + 0)
#define EINPROGRESS             (NET_ERR + 1)
#define EALREADY                (NET_ERR + 2)
#define ENOTSOCK                (NET_ERR + 3)
#define EDESTADDRREQ            (NET_ERR + 4)
#define EMSGSIZE                (NET_ERR + 5)
#define EPROTOTYPE              (NET_ERR + 6)
#define ENOPROTOOPT             (NET_ERR + 7)
#define EPROTONOSUPPORT         (NET_ERR + 8)
#define ESOCKTNOSUPPORT         (NET_ERR + 9)
#define EOPNOTSUPP              (NET_ERR + 10)
#define EAFNOSUPPORT            (NET_ERR + 11)
#define EADDRINUSE              (NET_ERR + 12)
#define EADDRNOTAVAIL           (NET_ERR + 13)
#define ENETDOWN                (NET_ERR + 14)
#define ENETUNREACH             (NET_ERR + 15)
#define ENETRESET               (NET_ERR + 16)
#define ECONNABORTED            (NET_ERR + 17)
#define ECONNRESET              (NET_ERR + 18)
#define ENOBUFS                 (NET_ERR + 19)
#define EISCONN                 (NET_ERR + 20)
#define ENOTCONN                (NET_ERR + 21)
#define ESHUTDOWN               (NET_ERR + 22)
#define ETIMEDOUT               (NET_ERR + 23)
#define ECONNREFUSED            (NET_ERR + 24)
#define EPFNOSUPPORT            (NET_ERR + 25)
#define EHOSTDOWN               (NET_ERR + 26)
#define EHOSTUNREACH            (NET_ERR + 27)

#define ENOURGENTDATA           (NET_ERR + 28)
#define ENOOOBDATA              (NET_ERR + 29)
#define  LAST_NET_ERR           ENOOOBDATA       /* last NET_ERR define */


/***** "socket" *****/

/* address families */

#define AF_UNSPEC   0   /* unspecified */
#define AF_INTRA    1   /* O/S-dependent path names */
#define AF_INET     2   /* ARPA Internet address */
#define AF_NS       3   /* Xerox Network Systems protocols */
#define AF_NBS      4   /* NBS protocols */
#define AF_CCITT    5   /* CCITT protocols, X.25 etc */
#define AF_SNA      6   /* IBM SNA */
#define AF_ARP      7   /* Address Resolution Protocol */
#define AF_KDB      8   /* Kernel DeBug Protocol */
#define AF_RARP     9   /* Reverse Address Resolution Protocol */
#define AF_1        10  /* customer expansion */
#define AF_2        11  /* "" */

/* link layer address families */

#define AF_LL       AF_2        /* last address family */
#define AF_PPP      (AF_LL+1)   /* Raw PPP */
#define AF_ETHER    (AF_LL+2)   /* Raw Ethernet */
#define AF_CMAN     (AF_LL+3)   /* Raw Serial Channel manager */
#define AF_IMPLINK  (AF_LL+4)   /* IMP "host at IMP" addresses */
#define AF_SLIP     (AF_LL+5)   /* SLIP link layers */
#define AF_DDN      (AF_LL+6)   /* DDN X.25 */
#define AF_TRN      (AF_LL+7)   /* Token Ring */
#define AF_FDDI     (AF_LL+8)   /* Fddi addresses */
#define AF_L3       (AF_LL+9)   /* " */
#define AF_L4       (AF_LL+10)  /* " */

#define MAX_AF      AF_L4   /* last link layer address family */
#define MIN_LLAF    AF_PPP  /* first link layer address family */
#define MAX_LLAF    MAX_AF  /* last link layer address family */
#define AF_UNIX     AF_INTRA    /* UNIX path names */


/* socket types */

#define SOCK_STREAM     1   /* virtual circuit */
#define SOCK_DGRAM      2   /* datagram */
#define SOCK_RAW        3   /* raw socket */
#define SOCK_RDM        4   /* reliable-delivered message */
#define SOCK_SEQPACKET  5   /* sequenced packet */
#define SOCK_PACK_EX    6   /* packet exchange non Berkeley */

/* level number for get/setsopt to apply to socket itself */

#define SOL_SOCKET      0xFFFF

/* socket options used with get/setsopt calls */

#define SO_DEBUG        0x1     /* turn on debugging */
#define SO_ACCEPTCONN   0x2     /* socket has had listen() */
#define SO_REUSEADDR    0x4     /* allow local address reuse */
#define SO_KEEPALIVE    0x8     /* keep connections alive */
#define SO_DONTROUTE    0x10    /* just use interface addresses */
#define SO_LINGER       0x80    /* linger on close if data is present */
#define SO_THROUGHPUT   0x100   /* large data transfer; handle efficiently */
#define SO_EXPEDITE     0x200   /* responsive (e.g. interactive) data
                                 * transfer; handle expeditiously */
#define SO_DONTLINGER   (~SO_LINGER)

#define MAXOPTVALEN     32      /* max. size of socket option value */

/* 4.2bsd send/recv standard flags */
#define MSG_OOB         0x1     /* process out of band information */
#define MSG_PEEK        0x2     /* peek at incoming message */
#define MSG_DONTROUTE   0x4     /* send without using routing tables */
/* 4.2bsd standard flag mask */
#define MSG_FLAGS       (MSG_OOB|MSG_PEEK|MSG_DONTROUTE)

/* NRC value added */
#define MSG_BLOCKING    0x8000  /* override any nonblocking state */
#define MSG_NONBLOCKING 0x4000  /* override any blocking state */
#define MSG_FDBROADCAST 0x2000  /* want full duplex broadcasts */
#define MSG_TRUNCATE    0x1000  /* truncate received packet */
#define MSG_URGENT      0x0800  /* urgent data */
#define MSG_REXMIT      0x0400  /* represents a retransmission; inform any
                        *  lower layers that there may be problems */
#define MSG_MASK        (MSG_FLAGS|MSG_BLOCKING|MSG_NONBLOCKING|MSG_FDBROADCAST|MSG_TRUNCATE|MSG_URGENT|MSG_REXMIT)


/* Address structures for all supported families */

typedef struct ipa {    /* DARPA Internet Protocol Address */
    a16     ip_port;    /* protocol port number */
    a32     ip_nethost; /* network/host address */
} ipa;

/* Typed socket addresses for all supported address families */

typedef struct sockaddr {   /* Generic socket address */
    short   sa_type;
    char    sa_data[22];
} saddr;


/* Socket option value structure */
typedef struct socketopt {
    int     so_optlen;  /* length of optdata area */
    char *  so_optdata; /* pointer to data */
} socketopt;


/****** "bsd_in" *****/

/*
 * Protocols
 */
#define IPPROTO_IP              0               /* dummy for IP */
#define IPPROTO_ICMP            1               /* control message protocol */
#define IPPROTO_IGMP            2               /* group control protocol */
#define IPPROTO_GGP             3               /* gateway^2 (deprecated) */
#define IPPROTO_TCP             6               /* tcp */
#define IPPROTO_EGP             8               /* exterior gateway protocol */
#define IPPROTO_PUP             12              /* pup */
#define IPPROTO_UDP             17              /* user datagram protocol */
#define IPPROTO_IDP             22              /* xns idp */
#define IPPROTO_HELLO           63              /* "hello" routing protocol */
#define IPPROTO_ND              77              /* UNOFFICIAL net disk proto */

#define IPPROTO_RAW             255             /* raw IP packet */
#define IPPROTO_MAX             256

/*
 * Port/socket numbers: network standard functions
 */
#define IPPORT_ECHO             7
#define IPPORT_DISCARD          9
#define IPPORT_SYSTAT           11
#define IPPORT_DAYTIME          13
#define IPPORT_NETSTAT          15
#define IPPORT_FTP              21
#define IPPORT_TELNET           23
#define IPPORT_SMTP             25
#define IPPORT_TIMESERVER       37
#define IPPORT_NAMESERVER       42
#define IPPORT_WHOIS            43
#define IPPORT_MTP              57

/*
 * Port/socket numbers: host specific functions
 */
#define IPPORT_TFTP             69
#define IPPORT_RJE              77
#define IPPORT_FINGER           79
#define IPPORT_TTYLINK          87
#define IPPORT_SUPDUP           95

/*
 * UNIX TCP sockets
 */
#define IPPORT_EXECSERVER       512
#define IPPORT_LOGINSERVER      513
#define IPPORT_CMDSERVER        514
#define IPPORT_EFSSERVER        520

/*
 * UNIX UDP sockets
 */
#define IPPORT_BIFFUDP          512
#define IPPORT_WHOSERVER        513
#define IPPORT_ROUTESERVER      520     /* 520+1 also used */

/*
 * Ports < IPPORT_RESERVED are reserved for
 * privileged processes (e.g. root).
 * Ports > IPPORT_USERRESERVED are reserved
 * for servers, not necessarily privileged.
 */
#define IPPORT_RESERVED         1024
#define IPPORT_USERRESERVED     5000

/*
 * Link numbers
 */
#define IMPLINK_IP              155
#define IMPLINK_LOWEXPER        156
#define IMPLINK_HIGHEXPER       158


/*
 * Internet address
 *      This definition contains obsolete fields for compatibility
 *      with SunOS 3.x and 4.2bsd.  The presence of subnets renders
 *      divisions into fixed fields misleading at best.  New code
 *      should use only the s_addr field.
 */
struct in_addr {
    union {
        struct { u8  s_b1,s_b2,s_b3,s_b4; } S_un_b;
        struct { u16 s_w1,s_w2; } S_un_w;
        u32     S_addr;
    } S_un;
#define s_addr  S_un.S_addr             /* should be used for all code */
#define s_host  S_un.S_un_b.s_b2        /* OBSOLETE: host on imp */
#define s_net   S_un.S_un_b.s_b1        /* OBSOLETE: network */
#define s_imp   S_un.S_un_w.s_w2        /* OBSOLETE: imp */
#define s_impno S_un.S_un_b.s_b4        /* OBSOLETE: imp # */
#define s_lh    S_un.S_un_b.s_b3        /* OBSOLETE: logical host */
};

/*
 * Definitions of bits in internet address integers.
 * On subnets, the decomposition of addresses to host and net parts
 * is done according to subnet mask, not the masks here.
 */
#define IN_CLASSA(i)            (((long)(i) & 0x80000000) == 0)
#define IN_CLASSA_NET           0xff000000
#define IN_CLASSA_NSHIFT        24
#define IN_CLASSA_HOST          0x00ffffff
#define IN_CLASSA_MAX           128

#define IN_CLASSB(i)            (((long)(i) & 0xc0000000) == 0x80000000)
#define IN_CLASSB_NET           0xffff0000
#define IN_CLASSB_NSHIFT        16
#define IN_CLASSB_HOST          0x0000ffff
#define IN_CLASSB_MAX           65536

#define IN_CLASSC(i)            (((long)(i) & 0xe0000000) == 0xc0000000)
#define IN_CLASSC_NET           0xffffff00
#define IN_CLASSC_NSHIFT        8
#define IN_CLASSC_HOST          0x000000ff

#define IN_CLASSD(i)            (((long)(i) & 0xf0000000) == 0xe0000000)
#define IN_MULTICAST(i)         IN_CLASSD(i)

#define IN_EXPERIMENTAL(i)      (((long)(i) & 0xe0000000) == 0xe0000000)
#define IN_BADCLASS(i)          (((long)(i) & 0xf0000000) == 0xf0000000)

#define INADDR_ANY              (u32)0x00000000

#define INADDR_LOOPBACK         (u32)0x7F000001
#define INADDR_BROADCAST        (u32)0xffffffff      /* must be masked */

#define IN_LOOPBACKNET          127                     /* official! */

/*
 * Socket address, internet style.
 */
struct sockaddr_in {
    short   sin_family;
    u16     sin_port;
    struct  in_addr sin_addr;
    char    sin_zero[8];
};

/*
 * Options for use with [gs]etsockopt at the IP level.
 */
#define IP_OPTIONS      1               /* set/get IP per-packet options */

#define IPVERSION       4


/***** "netioc" *****/

#ifndef SOIOCBASE
#define SOIOCBASE      ((u16)('s' << 8))
#endif

#ifndef SOIOCGNTTY
#define SOIOCGNTTY     (SOIOCBASE-20)  /* for get ntty */
#define SOIOCSNTTY     (SOIOCBASE-10)  /* for set ntty rlog/xns */
#define SOIOCSTEL      (SOIOCBASE-11)  /* for set ntty telnet/tcp */
#define SOIOCDTEL      (SOIOCBASE-12)  /* for daemon telnet/tcp */
#define SOIOCSRLOG     (SOIOCBASE-13)  /* for set ntty rlogin/tcp */
#define SOIOCDRLOG     (SOIOCBASE-14)  /* for daemon rlogin/tcp */
#define SOIOCRNTTY     (SOIOCBASE-15)  /* to reset attached tty */
#endif

#ifndef SOIOCATMARK
#define SOIOCATMARK     (SOIOCBASE-3)   /* for MSG_OOB */
#endif

#define SOIOCNTATTACH   (SOIOCBASE-4)   /* for net tty hookup */
#define SOIOCNTSETOUT   (SOIOCBASE-5)   /* screen output redirect */
#define SOIOCURG        (SOIOCBASE-6)   /* for MSG_OOB too */

#ifndef ENIOCBASE
#define ENIOCBASE       ((u16)('e' << 8))
#endif

#ifndef INPIOCBASE
#define INPIOCBASE      ((u16)('i' << 8))
#endif

#define INPIOCDOWNLOAD      (INPIOCBASE + 0)
#define INPIOCEXECUTE       (INPIOCBASE + 1)
#define INPIOCINITIALIZE    (INPIOCBASE + 2)
#define INPIOCPROBE         (INPIOCBASE + 3)
#define INPIOCDIDDLE        (INPIOCBASE + 4)

/* Values for raw ethernet ioctls.  These select the action inside
 * the link layer ioctl routine.
 */
#define ENIOCEND        (ENIOCBASE+10)
#define ENIOCNORMAL     ENIOCBASE       /* accept broadcast and specific */
#define ENIOCPROMISC    (ENIOCBASE+1)   /* accept all undamaged packets */
#define ENIOCALL        (ENIOCBASE+2)   /* accept ALL packets */
#define ENIOCRESET      (ENIOCBASE+3)
#define ENIOCWHATRU     (ENIOCBASE+4)   /* return device name from ndevsw */


/***** "ip" *****/

#define IP_MAXTTL   255 /* maximum packet life (4:15 minutes) */

/* IP protocol codes */
#define IP_IP       0   /* Internet Protocol (for use with get/setsopt only) */
#define IP_ICMP     1   /* Internet Control Message Protocol */
#define IP_GGP      3   /* Gateway-to-Gateway Protocol */
#define IP_TCP      6   /* Transaction Control Protocol */
#define IP_UDP      17  /* User Datagram Protocol */
#define IP_SNMP     57  /* Simple Network Management Protocol */

/* Bit fields of iph_saddr & iph_daddr */

/* #define INADDR_ANY  ((u32)0) */ /* unknown, or Berkeley broadcast address */
#define IP_BROADCAST    ((u32)0xFFFFFFFF) /* The real broadcast address */

#define MASK_A_CLASS    ((u32)0x80000000)
#define MASK_B_CLASS    ((u32)0xC0000000)
#define MASK_C_CLASS    ((u32)0xE0000000)
#define MASK_EX_CLASS   ((u32)0xE0000000)
#define A_CLASS         ((u32)0x00000000)
#define B_CLASS         ((u32)0x80000000)
#define C_CLASS         ((u32)0xC0000000)
#define EX_CLASS        ((u32)0xE0000000)
#define NET_A_CLASS     ((u32)0xFF000000)
#define NET_B_CLASS     ((u32)0xFFFF0000)
#define NET_C_CLASS     ((u32)0xFFFFFF00)
#define NET_EX_CLASS    ((u32)0xFFFFFFFF)
#define HOST_A_CLASS    ((u32)0x00FFFFFF)
#define HOST_B_CLASS    ((u32)0x0000FFFF)
#define HOST_C_CLASS    ((u32)0x000000FF)
#define HOST_EX_CLASS   ((u32)0xFFFFFFFF)

/* macro to compare two 'a32' pointers through given mask to see
 * if the network portion matches */
#define ip_nm(p1, p2, m) (((U32(*p1) ^ U32(*p2)) & m) == 0)

/* defines for IP level options */

#define IPO_CF          0x80    /* copy flag */
#define IPO_CLASS       0x60    /* option class */
#define IPO_NUMBER      0x1F    /* option number */

/* values for copy flag */

#define IPO_NCOPIED     0x00    /* option not copied */
#define IPO_COPIED      0x80    /* option copied */

/* values for option class */

#define IPO_CONTROL     0x00    /* control */
#define IPO_R1          0x20    /* reserved for future use */
#define IPO_DEBUG       0x40    /* debugging and measurement */
#define IPO_R2          0x60    /* reserved for future use */

/* values for option number */

#define IPO_EOL         0x00    /* end of option list */
#define IPO_NOP         0x01    /* 'u16' filler */
#define IPO_SECURITY    0x02    /* security:compartment:handling:tcc */
#define IPO_LSR         0x03    /* loose source and record route */
#define IPO_SSR         0x09    /* strict source and record route */
#define IPO_RR          0x07    /* record route */
#define IPO_SID         0x08    /* stream identifier */
#define IPO_TS          0x04    /* Internet timestamp */

/* IP socket security option */
typedef struct {
    a16 ipos_security;          /* security */
    a16 ipos_compartments;      /* compartments */
    a16 ipos_handling;          /* handling restrictions */
    a32 ipos_tcc;               /* transmission control code (m.s. 24 bits) */
} ipos_t;

/* Values for S field  of security option */

#define IPOS_UNCLASSIFIED   0x0000  /* Unclassified */
#define IPOS_CONFIDENTIAL   0xF135  /* Confidential */
#define IPOS_EFTO           0x789A  /* EFTO */
#define IPOS_MMMM           0xBC4D  /* MMMM */
#define IPOS_PROG           0x5E26  /* PROG */
#define IPOS_RESTRICTED     0xAF13  /* Restricted */
#define IPOS_SECRET         0xD788  /* Secret */
#define IPOS_TOP_SECRET     0x6BC5  /* Top Secret */
#define IPOS_R0             0x35E2  /* Reserved for future use */
#define IPOS_R1             0x9AF1  /*     "     "     "    "  */
#define IPOS_R2             0x4D78  /*     "     "     "    "  */
#define IPOS_R3             0x24BD  /*     "     "     "    "  */
#define IPOS_R4             0x135E  /*     "     "     "    "  */
#define IPOS_R5             0x89AF  /*     "     "     "    "  */
#define IPOS_R6             0xC4D6  /*     "     "     "    "  */
#define IPOS_R7             0xE26B  /*     "     "     "    "  */

/* IP socket timestamp option */

typedef struct ipot_t {
    a16 ipot_overflow_type; /* overflow/flag */
    a32 ipot_timestamps[9]; /* timestamp/address slots (max 9) */
} ipot_t;

/* bits defined in Internet timestamp option */
#define IPO_T_OVER  0xF0    /* Overflow mask */
#define IPO_T_TYPE  0x0F    /* Timestamp type mask */
#define IPO_T_TO    0x00    /* Timestamps only */
#define IPO_T_IT    0x01    /* Internet address/timestamp pairs */
#define IPO_T_PIT   0x03    /* Prespecified internet address/timestamp pairs */

/* values for IP level socket options */

#define IP_O_TOS    1       /* Type of service */ 
#define IP_O_FRAG   2       /* Fragmentation Control */
#define IP_O_MAXTTL 3       /* Maximum TTL value */ 
#define IP_O_MINTTL 4       /* Minimum TTL value */ 
#define IP_O_SECURE 5       /* security:compartment:handling:tcc */
#define IP_O_LSRR   6       /* loose source and record route */
#define IP_O_SSRR   7       /* strict source and record route */
#define IP_O_RR     8       /* record route */
#define IP_O_STREAM 9       /* stream identifier */
#define IP_O_TIME   10      /* Internet timestamp */

/* End of packed structures */

/***** "select" *****/

typedef struct sel {
    u16   se_inflags;     /* input/request flags */
    u16   se_outflags;    /* output/reply flags */
    i16   se_fd;          /* holds process local file id */
    i16   se_1reserved;   /* reserved */
    u32   se_user;        /* user-definable value */
    u32   se_2reserved;   /* reserved */
} sel;

/*
 * synchronous (via nselect) or asynchronous (via nselect or notify)
 * notification values.
 */
#define READ_NOTIFY         ((u16)0x1)   /* read data available */
#define WRITE_NOTIFY        ((u16)0x2)   /* write data space available */
#define ACCEPT_NOTIFY       ((u16)0x4)   /* connection request awaits */
#define CLOSE_NOTIFY        ((u16)0x8)   /* close down completed */
#define CONNECT_NOTIFY      ((u16)0x10)  /* connection completed */
#define EXCEPT_NOTIFY       ((u16)0x20)  /* exception has occurred */
#define RSHUTDOWN_NOTIFY    ((u16)0x40)  /* read direction shutdown by peer */
#define TIMEOUT_NOTIFY      ((u16)0x80)  /* request has timed out */
#define WSHUTDOWN_NOTIFY    ((u16)0x100) /* write direction shutdown by peer */
#define URGENT_NOTIFY       ((u16)0x200) /* urgent data available */
#define OOB_NOTIFY          ((u16)0x400) /* out of bounds data */

#define NSEL_BLOCK_MODE      0   /* nselect blocking mode */
#define NSEL_ASYNCH_MODE     1   /* nselect asynch mode */


/************/

/* socket function */
BEGIN_EXTERN_C              /* C++ support  99/9/8 */
extern  int     accept(int fd, saddr * addrp, int * addrlen, int * errp);
extern  int     bind(int fd, saddr * name, int namelen);
extern  int     blocking(int fd);
extern	int 	cabort (int sd);
extern  int     caccept(int fd, saddr * addrp, int * addrlen, int * errp);
extern  int 	connect(int fd, saddr * name, int namelen);
extern  int     getpeername(int sd, saddr * name, int * namelen);
extern  int     getsockname(int fd, saddr * saddrp, int * addrsize);
extern  int     getsopt(int sd, int level, int optname, char * optval
				, int * optlen);
extern  int 	getserr (int sd);
extern  int 	ip_bind(int s, int pref_sock, int * errp);
extern  int 	listen(int fd, int backlog);
extern  int     nonblocking(int fd);
extern  int     nselect(struct sel * selp, int cnt, int * waitp
			, int mode, unsigned long arg, int * errp);
extern	int     pioctl (int fd, int cmnd, char * addr, int len, u16 flags);
extern  int 	recv(int fd, char * buf, int buf_len, int flags, int * errp);
extern  int 	recvfrom(int fd, char * buf, int buflen, int flags
				, saddr * saddrp, int * addrlenp, int * errp);
extern  int 	reject(int fd);
extern  int     send(int fd, char * buf, int buf_len, int flags, int * errp);
extern  int     sendh(int fd, char * buf, int buf_len, int flags, char * hdr
				, int hdrlen, int * errp);
extern  int     sendto(int fd, char * buf, int buflen, int flags
				, saddr * saddrp, int saddrlen, int * errp);
extern  int     setsopt(int sd, int level, int optname, char * optval
			, int optlen);
extern	int     shutdown (int fd, int direction);
extern  int     socket(int af, int type, int protocol, int * errp);
extern  int     tcp_open(int pref_sock, ipa * far_address, int * errp);
extern  int     udp_open(int pref_sock, int * errp);

extern 	int     close(int fd) ;

extern  unsigned long   inet_addr (char * cp) ;
extern  char *          inet_ntoa (struct in_addr in) ;
END_EXTERN_C                /* C++ support  99/9/8 */
#define ntohs(C)  (u16)((((C)>>8)&0xFF)|(((C)<<8)&0xFF00))
#define htons(C)  (u16)((((C)>>8)&0xFF)|(((C)<<8)&0xFF00))
#define ntohl(C)  ((u32)(((((u32)C)>>24)&0xFF)|((((u32)C)>>8)&0xFF00)|((((u32)C)<<8)&0xFF0000)|((((u32)C)<<24)&0xFF000000)))
#define htonl(C)  ((u32)(((((u32)C)>>24)&0xFF)|((((u32)C)>>8)&0xFF00)|((((u32)C)<<8)&0xFF0000)|((((u32)C)<<24)&0xFF000000)))

/*Ή 96.04.04 */
#define INVALID_SEARCH      1                  /* IPAhXAhC */
#define NO_DNS_ENTRY        2                  /* DNST[oo^ĂȂ */
#define HOST_NOT_FOUND      3                  /* DNST[ỏȂ */
END_NAMESPACE_ZAURUS        /* C++ support  99/9/8 */
#endif /* _FUSIONH_ */
